-----------------------------------------------------------------------------------------------
-- Client Lua Script for BuffTelegraphs
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"

-----------------------------------------------------------------------------------------------
-- BuffTelegraphs Module Definition
-----------------------------------------------------------------------------------------------
local BuffTelegraphs = {} 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local CodeVersion = "2.5"
local EmpowerAmpId = 718

local EnumHasEmpoweringAmp = "2"
local EnumPowerLinkCast = "3"
local EnumFadeOutCast = "4"
local EnumPowerLinkCastWarn = "5"
local EnumFadeOutCastWarn = "6"
local EnumMissingEmpoweringAmp = "7"

local sin = math.sin
local cos =  math.cos
local rad = math.rad

getTime = os.clock
local classSprites = {
	[1] = "BTSprite:BTWarrior",
	[2] = "BTSprite:BTEngineer",
	[3] = "BTSprite:BTEsper",
	[4] = "BTSprite:BTMedic",
	[5] = "BTSprite:BTStalker",
	[7] = "BTSprite:BTSpellslinger",
	["T"] = "BTSprite:BTTank",
	["H"] = "BTSprite:BTHealer",
}
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function BuffTelegraphs:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
    return o
end

function BuffTelegraphs:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "BuffTelegraphs"
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end

local BuffTelegraphsInst = BuffTelegraphs:new()
BuffTelegraphsInst:Init()



-----------------------------------------------------------------------------------------------
-- BuffTelegraphs OnLoad
-----------------------------------------------------------------------------------------------
function BuffTelegraphs:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("BuffTelegraphs.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	Apollo.LoadSprites("BTSprite.xml");
end

-----------------------------------------------------------------------------------------------
-- BuffTelegraphs OnDocLoaded
-----------------------------------------------------------------------------------------------

function BuffTelegraphs:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.Overlay = Apollo.LoadForm(self.xmlDoc, "BuffTelegraphsForm","InWorldHudStratum", self)
	    --self.OptionsPopup = Apollo.LoadForm(self.xmlDoc, "OptionsPopUp",nil, self)
	    self.PopUp = Apollo.LoadForm(self.xmlDoc, "NewPopUp",nil, self)
--	    self.DebugPopUp = Apollo.LoadForm(self.xmlDoc, "DebugPopUp",nil, self)
		self.PowerLinkWindow = Apollo.LoadForm(self.xmlDoc, "PowerLinkWindow","FixedHudStratum", self)
		self.EmpoweringAuraWindow = Apollo.LoadForm(self.xmlDoc, "EmpoweringAuraWindow","FixedHudStratum", self)

		if self.Overlay == nil then
			Apollo.AddAddonErrorText(self, "Could not load the  window for some reason.")
			return
		end
		GeminiColor = Apollo.GetPackage("GeminiColor").tPackage
		
		self:AssembleOptionsMenu()
		self.PopUp:Show(false,true)
	    self.Overlay:Show(true, true)
		
		self.EmpoweringAuraWindow:Show(false, true)	
		self.PowerLinkWindow:Show(false, true)	

	
		self.Rez = self.PopUp:FindChild("RezAmount")
		self.RezSlide = self.PopUp:FindChild("RezSliderBar")

		self.PRez = self.PopUp:FindChild("PRezAmount")
		self.PRezSlide = self.PopUp:FindChild("PRezSliderBar")

		
		self.LineThicknessValue = self.PopUp:FindChild("LineThicknessValue")
		self.LineThicknessSlider = self.PopUp:FindChild("LineThicknessSliderBar")
		
		self.eButton = self.PopUp:FindChild("EnableButton")
	
		self.TitleMesssage = self.PopUp:FindChild("Title")

		self.CircleLimit = self.PopUp:FindChild("CircleLimitText")
		self.CircleLimitSlide = self.PopUp:FindChild("CircleLimitSliderBar")

		self.IconLimit = self.PopUp:FindChild("IconLimitText")
		self.IconLimitSlide = self.PopUp:FindChild("IconLimitSliderBar")
		
		self.IconSize = self.PopUp:FindChild("IconSizeText")
		self.IconSizeSlide = self.PopUp:FindChild("IconSizeSliderBar")
		
		self.CircleAlphaValue = self.PopUp:FindChild("CircleAlphaValue")		
		self.CircleAlphaSlide = self.PopUp:FindChild("CircleAlphaSliderBar")

		self.MinAlphaValue = self.PopUp:FindChild("MinAlphaValue")
		self.MinAlphaSlider = self.PopUp:FindChild("MinAlphaSliderBar")
	
--		self.debugt = self.DebugPopUp:FindChild("DebugTextBox")
		
		self.ShowWarriorBuffIconButton = self.PopUp:FindChild("ShowWarriorBuffIcon")
		self.ShowMedicBuffIconButton = self.PopUp:FindChild("ShowMedicBuffIcon")
		self.FrameSkipButton = self.PopUp:FindChild("Frameskip")
		
		self.canSendBuffData=false
		self.buffFrequency = 0.20
		self.DrawnCircles=0	
		self.DrawnIcons = 0 
		self.hasAmp = false
		
		if (self.tSettings == nil) then
			self:DefaultSettings()
			self:newOptions()
	    	--self.PopUp:Show(true, true)
		end
		
		self.PopUp:SetAnchorOffsets(self.tSettings.cOptionsPopupWindowL,self.tSettings.cOptionsPopupWindowT,self.tSettings.cOptionsPopupWindowL+self.PopUp:GetWidth(),self.tSettings.cOptionsPopupWindowT + self.PopUp:GetHeight())
		--self:FillWindow()	
		self:SetupOptions()
		--initialize variables
		self.watchList={}
		self.pixieCount=0
		self.frameCount=0
		self.player = nil
		self.playerClass = nil
		self.inParty = false
		self.canSendBuffData = false
		self.MStimer = 0
		self.startime = getTime()
		self.broadcastChannelName = nil
		self.spellCdTimer = nil

		Apollo.RegisterSlashCommand("bufftelegraphs", "newOptions", self)
		Apollo.RegisterSlashCommand("bt", "newOptions", self)
		Apollo.RegisterSlashCommand("btd", "ShowDebug", self)
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
		Apollo.RegisterEventHandler("BuffTrackerMenuClick", "newOptions", self)
		Apollo.RegisterEventHandler("AbilityBookChange", 		"OnAbilityBookChange", self)
		Apollo.RegisterEventHandler("Group_Join", 				"OnGroupJoin", self)
		Apollo.RegisterEventHandler("Group_Left", 				"OnGroupLeft", self)
		Apollo.RegisterEventHandler("Group_Add",				"OnGroupAdd", self)
		Apollo.RegisterEventHandler("Group_Changed",			"OnGroupChange", self)
--   		Apollo.RegisterEventHandler("Group_Updated",			"OnGroup_Updated", self)
		Apollo.RegisterEventHandler("Group_MemberPromoted",		"OnGroupMemberPromoted", self)		-- ( name, bSelf )
		Apollo.RegisterEventHandler("Group_Remove",				"OnGroupRemove", self)
   		Apollo.RegisterEventHandler("ChangeWorld",				"OnWorldChanged", self) 
	    Apollo.RegisterEventHandler("ChatZoneChange",			"OnWorldChanged", self)
		Apollo.RegisterEventHandler("CharacterCreated",			"OnCharacterCreated", self)
		Apollo.RegisterTimerHandler("AbilityBook_CacheTimer",	"UpdateAbilityBook", self)
		Apollo.RegisterTimerHandler("RestartTimer", "Restart", self)
		Apollo.RegisterEventHandler("Group_MemberFlagsChanged",	"GroupFlagsChanged", self)
		Apollo.RegisterTimerHandler("AbilityBook_CacheTimer",	"UpdateAbilityBook", self)
		Apollo.RegisterTimerHandler("SettingChange_CacheTimer",	"RefreshWatchList", self)

		
		
		if GroupLib.InRaid() or GroupLib.InGroup()then
			Apollo.CreateTimer("RestartTimer", 5, false)
		end
	end
end


-----------------------------------------------------------------------------------------------
-- BuffTelegraphs Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
-- on SlashCommand "/bt"
function BuffTelegraphs:newOptions()
	if not self.PopUp:IsVisible() then
		self.btimer = nil
		self.bpartytimer = nil
		self.PopUp:Invoke() -- show the window
	    Apollo.RemoveEventHandler("NextFrame", self)
	    Apollo.RegisterEventHandler("NextFrame", "OnNextFrame", self)
	    Apollo.RemoveEventHandler("VarChange_FrameCount", self)
		Apollo.RegisterEventHandler("VarChange_FrameCount", "OnVarFrame", self)
		self.btimer = ApolloTimer.Create(self.buffFrequency, true, "OnBuffTimer", self)
		self.bpartytimer = ApolloTimer.Create(0.50, true, "OnPartyTimer", self)
		
		self:ShowEditBuffIcons()
		
	else
		self:OnOptionsClose()	
	end
end

-- when the Cancel button is clicked
function BuffTelegraphs:OnOptionsClose( wndHandler, wndControl )
	if GroupLib.InRaid() or GroupLib.InGroup()then
		self:Restart()
	else
		self:Stop()
	end
	self.PopUp:Close() -- hide the window
end


function BuffTelegraphs:OnConfigure(sCommand, sArgs)
	if self.PopUp then
		self:newOptions()
	end
end




-- on timer
function BuffTelegraphs:OnBuffTimer()
	--Check for Medic of Warrior buffs on the player
	if GameLib.GetPlayerUnit() then
		self.player = GameLib.GetPlayerUnit()
		self:CheckPlayerBuffs()
	end
end


function BuffTelegraphs:OnPartyTimer()
	self.inParty = self.PopUp:IsShown()
	if GroupLib.InRaid() or GroupLib.InGroup()then
		self.inParty = true
		self:ExaminePartyMembers()
	end
end





-----------------------------------------------------------------------------------------------
-- BuffTelegraphs Instance
-----------------------------------------------------------------------------------------------
local BuffTelegraphsInst = BuffTelegraphs:new()
BuffTelegraphsInst:Init()


function BuffTelegraphs:Start()
    Apollo.RemoveEventHandler("NextFrame", self)
	Apollo.RegisterEventHandler("NextFrame", "OnNextFrame", self)
    Apollo.RemoveEventHandler("VarChange_FrameCount", self)
	Apollo.RegisterEventHandler("VarChange_FrameCount", "OnVarFrame", self)
	self.btimer = ApolloTimer.Create(self.buffFrequency, true, "OnBuffTimer", self)
	self.bpartytimer = ApolloTimer.Create(0.50, true, "OnPartyTimer", self)
end

function BuffTelegraphs:Stop()
	Apollo.RemoveEventHandler("NextFrame", self)
	Apollo.RemoveEventHandler("VarChange_FrameCount", self)
	if self.watchList then
		for i = 1, #self.watchList do
			if self.watchList[i].marker then
				self.watchList[i].marker:Destroy()
				self.watchList[i].marker = nil
			end
		end
	end
	self.watchList = {}
	self.spellt = nil
	self.player = nil
	self.btimer = nil
	self.bpartytimer = nil
	self.broadcastChannel = nil
	self.broadcastChannelName= nil
	self.watchList = {}
	self.inParty = false	
	if self.pMarker then self.pMarker:Destroy() self.pMarker = nil end
	
	self.Overlay:DestroyAllPixies()
	self.Overlay:DestroyChildren()
	self:HideAbilityIcons()
end


-- Main function called every frame
function BuffTelegraphs:OnNextFrame()

	if self.PopUp:IsVisible() then self:UpdateWindow() end
	if self.tSettings.frameSkip and (self.frameCount == 1)  then self.frameCount = 0  return
	elseif self.tSettings.frameSkip then
		self.frameCount = self.frameCount + 1
	end
    
	self.Overlay:DestroyAllPixies()
	self.tSettings.cEnabled = true
	if not self.tSettings.cEnabled then return end
	if not self.inParty then return end
	
		
	self.DrawnCircles=0
	self.DrawnIcons=0
	self.pixieCount = 0

	if self.player == nil then
		self.player = GameLib.GetPlayerUnit()
		self.playerClass = self.player:GetClassId()
		self:UpdateAbilityBook()
	end
	-- Main Fuction to draw circle	
	self:DrawWatchListCircles()
	self:DrawWatchIcons()

	
	if self.tSettings.bPlayerCircle then	
		local originPosition= self.player:GetPosition()
		local originPositionVector= Vector3.New(originPosition.x,originPosition.y,originPosition.z)
		local uface = self.player:GetFacing()
		local uangle = math.atan2(uface.x, uface.z)
		self:PlotCircle(originPositionVector,self.tSettings.nPlayerRange,"ff"..self.tSettings.cPlayerColor,uangle,true)
	end
	if self.tSettings.bPlayerMarker then
		local size = (self.tSettings.cIconSize or 24) * 0.75

		if self.pMarker == nil then
			self.pMarker = 	self:CreateMark(self.player,size,self.tSettings.cPlayerColor)
			self.pMarker:SetAnchorOffsets(-size,-size*2,size,0)
			self.pMarker:GetChildren()[1]:SetSprite("BTSprite:BTPlayer")
			self.pMarker:GetChildren()[1]:SetBGColor("cc"..self.tSettings.cPlayerColor)
		end
		--self.pMarker:SetWorldLocation(self.player:GetPosition())
	end


end

function BuffTelegraphs:CreateMark(unit,size,color)
	marker = Apollo.LoadForm(self.xmlDoc, "MarkFrame", "InWorldHudStratum", self)
	marker:GetChildren()[1]:SetBGColor("00"..color)
	marker:SetUnit(unit,0)
	marker:SetAnchorOffsets(-size,-size,size,size)
	return marker
end


function BuffTelegraphs:BuffTelegraphsDebug()
	local target = GameLib.GetTargetUnit() or self.player
	if target then
		self.inParty=true
		local classn = target:GetClassId()
    	if classn == GameLib.CodeEnumClass.Warrior and (self.tSettings.bWarriorCircle or self.tSettings.bWarriorMarker) then
			self:TryToAddToWatchList(target, self.tSettings.bWarriorCircle)
    	elseif classn == GameLib.CodeEnumClass.Medic and (self.tSettings.bMedicCircle or self.tSettings.bMedicMarker) then
			self:TryToAddToWatchList(target,self.tSettings.bMedicCircle)
    	elseif classn == GameLib.CodeEnumClass.Stalker and (self.tSettings.bStalkerCircle or self.tSettings.bStalkerMarker) then
			self:TryToAddToWatchList(target,self.tSettings.bStalkerCircle)
    	elseif classn == GameLib.CodeEnumClass.Esper and (self.tSettings.bEsperCircle or self.tSettings.bEsperMarker) then
			self:TryToAddToWatchList(target,self.tSettings.bEsperCircle)
    	elseif classn == GameLib.CodeEnumClass.Spellslinger and (self.tSettings.bSpellslingerCircle or self.tSettings.bSpellslingerMarker) then
			self:TryToAddToWatchList(target,self.tSettings.bSpellslingerCircle)
    	elseif classn == GameLib.CodeEnumClass.Engineer and (self.tSettings.bEngineerCircle or self.tSettings.bEngineerMarker) then
			self:TryToAddToWatchList(target,self.tSettings.bEngineerCircle)
		end
	end
end


function BuffTelegraphs:TryToAddToWatchList(unit,bType,role)
	if self.player == nil then return end
--	unitid = unit:GetId()
	if unit == self.player then return end
	
	if not self:IsOnWatchList(unit) then
		self:AddToWatchList(unit,bType,role)
	end
end


function BuffTelegraphs:ExaminePartyMembers()
	--self.WatchBox:SetText(#self.watchList .. " on Watchlist")
    for i = 1, GroupLib.GetMemberCount(), 1 do
		local gmember = GroupLib.GetGroupMember(i)

--[[        if gmember.bIsOnline then
			if gmember.bIsLeader then
				-- join a custom channel to send data cross client.
				if self.broadcastChannelName == nil then
					self.broadcastChannelName = tostring(gmember.strCharacterName .. "BTChannel")
					self.broadcastChannel = "valid" --ICCommLib.JoinChannel(self.broadcastChannelName, "OnChanMessage", self)
					--self.broadcastChannel = ICCommLib.JoinChannel(self.broadcastChannelName, "OnChanMessage", self)
				end
			end
]]			
			local member = GroupLib.GetUnitForGroupMember(i)
			if member then
				if member ~= self.player then
					local memberID = member:GetId()
					local classn = member:GetClassId()
					
					
					if self.tSettings.bTankMarker and (gmember.bTank or gmember.bMainTank) then
						self:TryToAddToWatchList(member, false,"T")
					elseif self.tSettings.bHealerMarker and gmember.bHealer then
						self:TryToAddToWatchList(member, false,"H")
			    	elseif classn == GameLib.CodeEnumClass.Warrior and (self.tSettings.bWarriorCircle or self.tSettings.bWarriorMarker) then
						self:TryToAddToWatchList(member, self.tSettings.bWarriorCircle)
					elseif classn == GameLib.CodeEnumClass.Medic and (self.tSettings.bMedicCircle or self.tSettings.bMedicMarker) then
						if gmember.bHealer and not self.tSettings.cIgnoreMedicHealers then
							self:TryToAddToWatchList(member,self.tSettings.bMedicCircle)
						elseif not gmember.bHealer then
							self:TryToAddToWatchList(member,self.tSettings.bMedicCircle)
						end
					elseif classn == GameLib.CodeEnumClass.Stalker and (self.tSettings.bStalkerCircle or self.tSettings.bStalkerMarker) then
						self:TryToAddToWatchList(member,self.tSettings.bStalkerCircle)
					elseif classn == GameLib.CodeEnumClass.Esper and (self.tSettings.bEsperCircle or self.tSettings.bEsperMarker) then
						self:TryToAddToWatchList(member,self.tSettings.bEsperCircle)
					elseif classn == GameLib.CodeEnumClass.Spellslinger and (self.tSettings.bSpellslingerCircle or self.tSettings.bSpellslingerMarker) then
						self:TryToAddToWatchList(member,self.tSettings.bSpellslingerCircle)
					elseif classn == GameLib.CodeEnumClass.Engineer and (self.tSettings.bEngineerCircle or self.tSettings.bEngineerMarker) then
						self:TryToAddToWatchList(member,self.tSettings.bEngineerCircle)
					end
				end	
			end
		end
	end
--end




function BuffTelegraphs:DrawlineS(startPoint,endPoin,bAlpha,bColor)
	self.Overlay:AddPixie( { bLine = true, fWidth = 2,  cr = {a=bAlpha,r=bColor.x,g=bColor.y,b=bColor.z}, loc = { fPoints = {0,0,0,0}, nOffsets = { startPoint.x, startPoint.y, endPoint.x, endPoint.y } }    } )
end

function BuffTelegraphs:OnUnitGroupChanged(unit)
end

function BuffTelegraphs:ResetListMembers()
	self:ResetWatchList()
end

function BuffTelegraphs:GroupFlagsChanged(i)
	local member = GroupLib.GetUnitForGroupMember(i)
	if member then
		--find the watchlist unit
		for j = 1, #self.watchList do
			if self.watchList[j].unit == member then
			Apollo.StopTimer("SettingChange_CacheTimer")
			Apollo.CreateTimer("SettingChange_CacheTimer", 2, false)
--			self.watchList[i].marker:Destroy()
--			self.watchList[i] = nil
		end
	end
end
end

function BuffTelegraphs:RemoveFromWatchList(unitid,classn)
	for i = 1, #self.watchList do
		if self.watchList[i].id == unitid then
		   table.remove( self.watchList, i )
			return
		end
	end
end


function BuffTelegraphs:AddToWatchList(unit,bType,role)
	local count = #self.watchList + 1
	self.watchList[count] = {}
	self.watchList[count].unit = unit
	self.watchList[count].id = unit:GetId()
	self.watchList[count].class = role or unit:GetClassId()
	self.watchList[count].name = unit:GetName()
	self.watchList[count].buffState = "no data"
	self.watchList[count].bAlpha = 10
	self.watchList[count].bOnScreen = false
	self.watchList[count].bCircles = bType
	self.watchList[count].marker =	self:CreateMark(unit,self.tSettings.cIconSize,"00000000")
	self.watchList[count].marker:GetChildren()[1]:SetSprite(classSprites[self.watchList[count].class])
end


function BuffTelegraphs:IsOnWatchList(unit)
	if #self.watchList==0 then return false	end
	
	for i = 1, #self.watchList do
		if self.watchList[i].unit == unit then
			return true
		end
	end
	return false
end

function BuffTelegraphs:DrawWatchIcons()
end

function BuffTelegraphs:DrawWatchListCircles()
	for i = 1, #self.watchList do
		if self.DrawnCircles < self.tSettings.cMaxCircles and self.DrawnIcons < self.tSettings.cMaxIcons then
				if self.watchList[i]== nil then Print("Error in WatchList") break end
				local target =	GameLib.GetUnitById(self.watchList[i].id)
			if target then
				if not target:IsDead() then
					self:DrawCircleRemix(target,self.watchList[i])
				end
			end
		end
	end
end


function BuffTelegraphs:ResetWatchList()
	for i = 1, #self.watchList do
		if self.watchList[i].marker then
			self.watchList[i].marker:Destroy()
			self.watchList[i].marker = nil
		end

	end

	self.watchList = {}
	self:ExaminePartyMembers()
end


function BuffTelegraphs:Restart()
	self:Stop()
	self:Start()
end


function BuffTelegraphs:OnGroupAdd()
	self:ResetWatchList()
end


function BuffTelegraphs:OnGroupJoin()
	self:Restart()
end


function BuffTelegraphs:OnGroupRemove()
	self:ResetWatchList()
end


function BuffTelegraphs:OnGroupLeft()
	self:Stop()
end


function BuffTelegraphs:OnWorldChanged()
	self:Stop()
	self.player = GameLib.GetPlayerUnit()
		if GroupLib.InRaid() or GroupLib.InGroup()then
			Apollo.CreateTimer("RestartTimer", 5, false)
	end
end


function BuffTelegraphs:StopAllTimers()
	self.btimer= nil
	self.bpartytimer= nil
	self.abilitytimer= nil
	self.spellCdTimer= nil
end


function BuffTelegraphs:StartAllTimers()
	self.btimer = ApolloTimer.Create(self.buffFrequency, true, "OnBuffTimer", self)
	self.bpartytimer = ApolloTimer.Create(0.50, true, "OnPartyTimer", self)
--	self.abilitytimer= nil
--	self.spellCdTimer= nil
end



function BuffTelegraphs:OnCharacterCreated()
	self.player = GameLib.GetPlayerUnit()
	self.playerClass = self.player:GetClassId()
end


function BuffTelegraphs:ShowDebug()
	if not self.DebugPopUp:IsVisible() then
		self.DebugPopUp:Invoke() -- show the window
	else
		self.DebugPopUp:Close()	
	end
end


function BuffTelegraphs:GetSpellByName(spellName)
	local abilities = AbilityBook.GetAbilitiesList()
	if abilities == nil then return end
		for _, v in pairs(abilities) do
			if v.bIsActive and v.nCurrentTier and v.tTiers then
				local tier = v.tTiers[v.nCurrentTier]
				if tier then
					local tierspellname = tier.splObject:GetName()
					if tierspellname == spellName then
						return tier.splObject
					end
				end
			end
		end
	return
end


function BuffTelegraphs:PlayerHasAmp(amp)
	local tEldanAugmentationData = AbilityBook.GetEldanAugmentationData(AbilityBook.GetCurrentSpec())
	for idx = 1, #tEldanAugmentationData.tAugments do
		local tAmp = tEldanAugmentationData.tAugments[idx]
		if tAmp.eEldanAvailability == AbilityBook.CodeEnumEldanAvailability.Activated then
			if tAmp.nId == amp then
				return tAmp.nId
			end
		end
	end
	return
end


--Scan for relevant buffs on the player
function BuffTelegraphs:CheckPlayerBuffs()
	self.player = GameLib.GetPlayerUnit()

	if not self.player then return end
	if not self.inParty then
		self.EmpoweringAuraWindow:Show(false)
		self.PowerLinkWindow:Show(false)
		return
	end

	self.hasWarriorBuff = false
	self.hasMedicBuff = false
	

	--Very costly call, this is why the check is on a timer
	local tBuffs = self.player:GetBuffs().arBeneficial
	if not tBuffs then return end
	for k, v in pairs(tBuffs) do
		local spellname = v.splEffect:GetName()
	    if  spellname == "Empowering Aura" then
			self.hasMedicBuff=true
		end
        if spellname == "Power Link" then
			self.hasWarriorBuff=true
		end
	end
	
	
	if self.PopUp:IsShown() then
		self.EmpoweringAuraWindow:Show(self.tSettings.cShowMedicBuffIcon)
		self.PowerLinkWindow:Show(self.tSettings.cShowWarriorBuffIcon)
	else
		if self.tSettings.cShowMedicBuffIcon then
			self.EmpoweringAuraWindow:Show(self.hasMedicBuff)
		end
		if self.tSettings.cShowWarriorBuffIcon then
			self.PowerLinkWindow:Show(self.hasWarriorBuff)
		end
	end
	
	if self.canSendBuffData then
		--self:SendBuffData()
	end
end


function BuffTelegraphs:SendBuffData()
	if self.spellt == nil then
		self:UpdateAbilityBook()
		return
	end
		--Check player class and send data if needed.
	if self.playerClass == GameLib.CodeEnumClass.Warrior then
		local spellCDR = self.spellt:GetCooldownRemaining()
		local spellCDT = self.spellt:GetCooldownTime()
		
		-- spell is ready for use, no need to send anything
		if	spellCDR == 0 then return end
		
		--Check to see if spell was cast			
		if spellCDR < spellCDT and 	not self.SpellWasCast then
			self.SpellWasCast = true
			self:SendMessageToGroup(EnumPowerLinkCast)
			return
		end
		if spellCDR < 5 and spellCDR ~= 0 then
			self.SpellWasCast = false
			self.canSendBuffData= false
			self:SendMessageToGroup(EnumPowerLinkCastWarn)
	        self.spellCdTimer = ApolloTimer.Create(5.0, false, "OnSpellCdTimer", self)
	end
	elseif self.playerClass == GameLib.CodeEnumClass.Medic then
		if self.hasAmp then
			self.canSendBuffData = false
			self:SendMessageToGroup(EnumHasEmpoweringAmp)
	        self.spellCdTimer = ApolloTimer.Create(15.0, false, "OnSpellCdTimer", self)
		else
			self.canSendBuffData = false
			self:SendMessageToGroup(EnumMissingEmpoweringAmp)
	        self.spellCdTimer = ApolloTimer.Create(15.0, false, "OnSpellCdTimer", self)
		end
	end 
end


function BuffTelegraphs:OnSpellCdTimer()
	self.canSendBuffData = true
end


function BuffTelegraphs:SendMessageToGroup(Msg,Msg2)
	if self.broadcastChannel then
		local outMsg= {}
		outMsg.strName = self.player:GetName()
		outMsg.strMsg = Msg
		outMsg.strClass = self.playerClass
		outMsg.strChannelName = self.broadcastChannelName
		self:OnChanMessage(18,outMsg)
		self.broadcastChannel:SendMessage(outMsg)
	end
end


function BuffTelegraphs:OnChanMessage(tChan, tMsg)
--Makes sure to only bother with messages from those in your groupchannel
	if tMsg.strChannelName == self.broadcastChannelName then
		if tMsg.strMsg ~= nil then
			self:MsgFromPlayer(tMsg.strName, tMsg.strMsg)
		end
	end
end


function BuffTelegraphs:MsgFromPlayer(strName, strMsg)

	if self.broadcastChannel == nil then return end

	if strMsg == EnumPowerLinkCast then
--		self.debugt:SetText(self.broadcastChannelName .. "\n" .."Power Link Cast by "..strName)
	elseif strMsg == EnumPowerLinkCastWarn then 
--		self.debugt:SetText(self.broadcastChannelName .. "\n" .."Power Link on "..strName.." ready in 5 Seconds")
	elseif strMsg == EnumHasEmpoweringAmp then 
--		self.debugt:SetText(self.broadcastChannelName .. "\n" .."Empowering Aura active on"..strName)
	elseif strMsg == EnumMissingEmpoweringAmp then 
--		self.debugt:SetText(self.broadcastChannelName .. "\n" .."Empowering Aura Missing on"..strName)
--	elseif strMsg == EnumFadeOutCastWarn then 
--		self.debugt:SetText(self.broadcastChannelName .. "\n" .."Projected Spirit on "..strName.." ready in 5 Seconds")
-- 	elseif strMsg == EnumFadeOutCast then 
--		self.debugt:SetText(self.broadcastChannelName .. "\n" .."Projected Spirit Cast by "..strName)
	end
	self:SetEventOnWatchList(strName, strMsg)
end


function BuffTelegraphs:SetEventOnWatchList(strName, strMsg)
	for i = 1, #self.watchList do
		if self.watchList[i].name == strName then
			self.watchList[i].buffState = strMsg
			return
		end
	end
end


function BuffTelegraphs:OnAbilityBookChange()
	self.canSendBuffData=false
	--Timer is needed because of multiple calls to the OnAbilityBookChange() callback
	Apollo.CreateTimer("AbilityBook_CacheTimer", 1, false)
end

	--GetSpellByName is a very costly call
function BuffTelegraphs:UpdateAbilityBook()
	self.player = GameLib.GetPlayerUnit()
	if self.player == nil then
		Apollo.CreateTimer("AbilityBook_CacheTimer", 1, false)
		return
	end
	--remove the timer for reseting canSendBuffData
	self.spellCdTimer = nil

	self.playerClass = self.player:GetClassId()
	if self.playerClass == GameLib.CodeEnumClass.Warrior then
		self.spellt =  self:GetSpellByName("Power Link")
	elseif self.playerClass == GameLib.CodeEnumClass.Medic then
		self.spellt =  EmpowerAmpId --This is just put here so it passes tests later
		self.hasAmp =  self:PlayerHasAmp(EmpowerAmpId)
	-- This is just here for development as I play an esper and need to test
--	elseif self.playerClass == GameLib.CodeEnumClass.Esper then
--		self.spellt =  self:GetSpellByName("Projected Spirit")
	end
	if self.spellt ~= nil then
		self.canSendBuffData = true
	end
end


function BuffTelegraphs:OnGroupMemberPromoted()
	self.broadcastChannelName = nil
end


function BuffTelegraphs:UpdateWindow()
	self:BuffTelegraphsDebug()
end


function BuffTelegraphs:HideAbilityIcons()
	if self.PopUp:IsShown() then return end
	self.EmpoweringAuraWindow:Show(false)
	self.PowerLinkWindow:Show(false)
	self.EmpoweringAuraWindow:SetStyle("Moveable",false)
	self.EmpoweringAuraWindow:SetStyle("IgnoreMouse", true)
	self.EmpoweringAuraWindow:SetStyle("Sizable", false)
	self.PowerLinkWindow:SetStyle("Moveable",false)
	self.PowerLinkWindow:SetStyle("IgnoreMouse", true)
	self.PowerLinkWindow:SetStyle("Sizable", false)
end


function BuffTelegraphs:OnInterfaceMenuListHasLoaded()
		Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "BuffTelegraphs", {"BuffTrackerMenuClick", "", ""})
end


function BuffTelegraphs:OnSave(eLevel)
	self:StopAllTimers()
	
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	
	local tsave= {}
	self:NewColorsSet()
	tsave.tSettings = self.tSettings
	--[[
	tsave.cSaveVersion = self.tSettings.cSaveVersion
	tsave.cResolution = self.tSettings.cResolution
	tsave.cEnabled = self.tSettings.cEnabled
	tsave.cMaxCircles = self.tSettings.cMaxCircles
	tsave.cIgnoreMedicHealers = self.tSettings.cIgnoreMedicHealers
	tsave.cTrackWarriors = self.tSettings.cTrackWarriors
	tsave.cTrackMedics = self.tSettings.cTrackMedics
	tsave.cCircleAlpha = self.tSettings.cCircleAlpha
	tsave.cMinAlpha = self.tSettings.cMinAlpha
	tsave.cLineThickness = self.tSettings.cLineThickness
	tsave.cShowWarriorBuffIcon = self.tSettings.cShowWarriorBuffIcon
	tsave.cShowMedicBuffIcon = self.tSettings.cShowMedicBuffIcon
	tsave.frameSkip = self.tSettings.frameSkip
	]]
	
	local l,t,r,b = self.PowerLinkWindow:GetAnchorOffsets()
	tsave.tSettings.cPowerLinkWindowL = l
	tsave.tSettings.cPowerLinkWindowT = t
	tsave.tSettings.cPowerLinkWindowR = r
	tsave.tSettings.cPowerLinkWindowB = b
	
	local l,t,r,b = self.EmpoweringAuraWindow:GetAnchorOffsets()
	tsave.tSettings.cEmpoweringAuraWindowL = l
	tsave.tSettings.cEmpoweringAuraWindowT = t
	tsave.tSettings.cEmpoweringAuraWindowR = r
	tsave.tSettings.cEmpoweringAuraWindowB = b
	
	local l,t,r,b = self.PopUp:GetAnchorOffsets()
	tsave.tSettings.cOptionsPopupWindowL = l
	tsave.tSettings.cOptionsPopupWindowT = t
	tsave.tSettings.cOptionsPopupWindowR = r
	tsave.tSettings.cOptionsPopupWindowB = b
		
    return tsave
end

function BuffTelegraphs:DefaultTable()
	local defaultsettings ={
	cResolution=6,
	cPResolution=8,
	cEnabled=true,
	cMaxCircles=5,
	cTrackWarriors=true,
	cTrackMedics=true,
	cIgnoreMedicHealers=true,
	cMinAlpha=20,
	frameSkip = false,
	cCircleAlpha = 70,
	cLineThickness = 2,
	cShowWarriorBuffIcon = true,
	cShowMedicBuffIcon = true,
	cEmpoweringAuraWindowL = 668,
	cEmpoweringAuraWindowT = 221,
	cEmpoweringAuraWindowR = 728,
	cEmpoweringAuraWindowB = 281,
	cPowerLinkWindowL = 668,
	cPowerLinkWindowT = 293,
	cPowerLinkWindowR = 728,
	cPowerLinkWindowB = 353,
	cOptionsPopupWindowL= 50,
	cOptionsPopupWindowT = 25,
	cHealerColor = "ff00ff00",
	cEsperColor = "ff1591DB",
	cMedicColor = "ffffE757",
	cStalkerColor = "FFD23EF4",
	cWarriorColor = "FFF54F4F",
	cSpellslingerColor = "FF98C723",
	cEngineerColor = "ffEFAB48",
	cTankColor = "ff4f738e",
	cPlayerColor = "ffffffff",
	cIconSize = 24,
	nStalkerRange = 12,
	nEsperRange = 15,
	nWarriorRange = 6,
	nSpellslingerRange = 8,
	nMedicRange = 10,
	nEngineerRange = 15,
	nPlayerRange = 10,
	bWarriorCircle = true,
	bMedicCircle = true,
	bEsperCircle = false,
	bStalkerCircle = false,
	bEngineerCircle = false,
	bSpellslingerCircle = false,
	bPlayerCircle = false,
	
	bWarriorMarker = false,
	bMedicMarker = false,
	bEsperMarker = false,
	bStalkerMarker = false,
	bEngineerMarker = false,
	bSpellslingerMarker = false,
	bPlayerMarker = false,
	
	cMaxIcons = 7
	
	}
	return defaultsettings

end

function BuffTelegraphs:OnRestore(eLevel, saveData)
	if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then return end
	
	self.tSettings = {}			
	local defaultsettings = self:DefaultTable()
		
	if saveData.tSettings then
	   	self.tSettings=saveData.tSettings
	end

	for k,v in pairs(defaultsettings) do
	   	if self.tSettings[k] == nil then
			 self.tSettings[k] = v
		end
	end
end


function BuffTelegraphs:DefaultSettings()
	self.tSettings = nil
	self.tSettings = self:DefaultTable()
	self:SetupOptions()
	self:RefreshWatchList()
end



function BuffTelegraphs:OnVarFrame()
	if not self.player then return end
	local curtime = getTime()
	self.MStimer = (curtime - self.startime)
	self.startime = curtime
	local pPos = self.player:GetPosition()
	local pVec = Vector3.New(pPos.x,pPos.y,pPos.z)
	

	for i = 1, #self.watchList do
		if self.watchList[i].unit:IsValid() and not self.watchList[i].unit:IsDead() then
			local uPos = self.watchList[i].unit:GetPosition()
			local uVec = Vector3.New(uPos.x,uPos.y,uPos.z)
			
			local length	= (pVec - uVec):Length()
			self.watchList[i].vLength = length

		    local sPos = GameLib.GetUnitScreenPosition(self.watchList[i].unit)
			if self.watchList[i].marker and sPos.bOnScreen and not self.watchList[i].bOnscreen and self.watchList[i].unit:IsOccluded() then
				self.watchList[i].bOnscreen = false
			end
			

			if self.watchList[i].marker then
			self.watchList[i].marker:Show(self.watchList[i].bOnscreen)
			end
			
			self.watchList[i].bOnscreen = sPos.bOnScreen
			
			if sPos.bOnScreen and self.watchList[i].unit:IsOccluded() then
			end
			
			
			local classn = self.watchList[i].class
			local bAlpha
			self.watchList[i].dist = 10 -- default distance
			-- Time needed to go from 0.0 to 1.0 alpha
			local bFadeTime = 500 --500ms  default
	
			local cColor = "ffffff"
			local bAlphaTarget = self.tSettings.cCircleAlpha
			if classn == "T" then
					cColor = self.tSettings.cTankColor
			elseif classn == "H" then			
					cColor = self.tSettings.cHealerColor
		    elseif classn == GameLib.CodeEnumClass.Medic then
				-- If medic doesn't have the amp, hide them.
		
					cColor = self.tSettings.cMedicColor
				self.watchList[i].dist=self.tSettings.nMedicRange
		
				if self.hasMedicBuff then
					bFadeTime = 250
					bAlphaTarget = self.tSettings.cMinAlpha
				else
					bFadeTime= 250
				end
		    elseif classn == GameLib.CodeEnumClass.Warrior then

				cColor = self.tSettings.cWarriorColor
				self.watchList[i].dist=self.tSettings.nWarriorRange
		
				if self.watchList[i].buffState == EnumPowerLinkCastWarn then
					bFadeTime = 3000
				elseif self.watchList[i].buffState == EnumPowerLinkCast or self.hasWarriorBuff then
					bAlphaTarget = self.tSettings.cMinAlpha
					bFadeTime = 500
				end
		    elseif classn == GameLib.CodeEnumClass.Esper then
				cColor = self.tSettings.cEsperColor
				self.watchList[i].dist=self.tSettings.nEsperRange
		    elseif classn == GameLib.CodeEnumClass.Stalker then
				cColor = self.tSettings.cStalkerColor
				self.watchList[i].dist=self.tSettings.nStalkerRange
		    elseif classn == GameLib.CodeEnumClass.Engineer then
				cColor = self.tSettings.cEngineerColor
				self.watchList[i].dist=self.tSettings.nEngineerRange
			elseif classn == GameLib.CodeEnumClass.Spellslinger then
				cColor = self.tSettings.cSpellslingerColor
				self.watchList[i].dist=self.tSettings.nSpellslingerRange
			end
			-- We want to ramp the alpha based on time, not frames
			local bAlphaMult = 100 * self.MStimer * (1000 / bFadeTime)
			--	local checkAlpha = unitInfo.bAlpha
		    if 	self.watchList[i].bAlpha < bAlphaTarget  then

				self.watchList[i].bAlpha  = self.watchList[i].bAlpha + (bAlphaMult)
				if self.watchList[i].bAlpha > bAlphaTarget then 
					self.watchList[i].bAlpha = bAlphaTarget
				end
		    elseif 	self.watchList[i].bAlpha > bAlphaTarget then
				self.watchList[i].bAlpha = self.watchList[i].bAlpha - (bAlphaMult)
				if self.watchList[i].bAlpha < bAlphaTarget then 
					self.watchList[i].bAlpha = bAlphaTarget
				end
			end
			-- final color
			if self.watchList[i].vLength > 100 then self.watchList[i].bAlpha = 00 end
			local finalAlpha = string.format("%02x",self.watchList[i].bAlpha*2.55)
			local finalcolor = finalAlpha .. cColor	
			
			if self.watchList[i].marker == nil then
				self.watchList[i].marker =	self:CreateMark(self.watchList[i].unit,self.tSettings.cIconSize,finalcolor)
				self.watchList[i].marker:GetChildren()[1]:SetSprite(classSprites[self.watchList[i].class])
			end
			if self.watchList[i].bColor ~=  finalcolor and sPos.bOnScreen then
				if self.watchList[i].unit:IsValid() and self.watchList[i].marker and self.watchList[i].marker:GetChildren() then
					self.watchList[i].marker:GetChildren()[1]:SetBGColor(finalcolor)
				end
			end
			
			if not sPos.bOnScreen then
				self.watchList[i].marker:Show(false,true)

			end			
			self.watchList[i].bColor =  finalcolor
		else
			if self.watchList[i].marker then
				self.watchList[i].marker:Destroy()
				self.watchList[i].marker = nil
			end
		end
	end
	
     table.sort(self.watchList, function(a, b) return (b.vLength or 0) < (a.vLength or 0) end)
end

--this function decides mostly the alpha of drawn circles based on player / group member buffs
function BuffTelegraphs:DrawCircleRemix(unit, unitInfo)
				
	if not unitInfo.bOnscreen then  return end
	if unitInfo.buffState == EnumMissingEmpoweringAmp then
		return
	end

	local originPosition= unit:GetPosition()
	
	if unitInfo.bCircles and self.DrawnCircles < self.tSettings.cMaxCircles then
		local originPositionVector= Vector3.New(originPosition.x,originPosition.y,originPosition.z)
		local uface = unit:GetFacing()
		local uangle = math.atan2(uface.x, uface.z)
		self:PlotCircle(originPositionVector,unitInfo.dist,unitInfo.bColor,uangle)
	elseif unitInfo.marker then	
		unitInfo.marker:SetWorldLocation(unitInfo.unit:GetPosition())
	end
end


function BuffTelegraphs:PlotCircle(originPositionVector, dist, color,uangle,player)
	--if dist == nil then print("failed") end
	dist = dist or 10
	local previousVector=Vector3.New(originPositionVector.x+dist*sin(uangle), originPositionVector.y, originPositionVector.z+dist*cos(uangle))
	local startPoint = GameLib.WorldLocToScreenPoint(previousVector)
	local pPoints = {0,0,0,0}
	
	local rez = self.tSettings.cResolution
	if player then rez = self.tSettings.cPResolution end
	
	for i = 1, rez do 
		local rotation_point = uangle + rad(-360 / (rez/(i)))

		local currentVector = Vector3.New(originPositionVector.x+dist*sin(rotation_point), originPositionVector.y, originPositionVector.z+dist*cos(rotation_point))
		local endPoint = GameLib.WorldLocToScreenPoint(currentVector)

		self.Overlay:AddPixie( { bLine = true, fWidth = self.tSettings.cLineThickness,  cr = color, loc = { fPoints = pPoints, nOffsets = { startPoint.x, startPoint.y, endPoint.x, endPoint.y } }    } )

		previousVector=currentVector
		startPoint = endPoint
	end
	if not player then
		self.DrawnCircles = self.DrawnCircles + 1 
	end
end




function BuffTelegraphs:AssembleOptionsMenu()
	--if self.PopUp == nil then return end
	
	self.CategoryList = self.PopUp:FindChild("CategoryList")
	self.OptionsList = self.PopUp:FindChild("Options")
	self:MakeCategory("General Options","SliderOptions")
	self:MakeCategory("Tanks / Healers","TanksHealersOptions")

	self:MakeCategory("Warriors","WarriorOptions")
	self:MakeCategory("Medics","MedicOptions")
	self:MakeCategory("Engineers","EngineerOptions")
	self:MakeCategory("Stalkers","StalkerOptions")
	self:MakeCategory("Spellslingers","SpellslingerOptions")
	self:MakeCategory("Espers","EsperOptions")
	self:MakeCategory("Player","PlayerOptions")
	
	self.CategoryList:ArrangeChildrenVert()
	--self.CategoryList:SetRadioSel("KBCategories", 1)
	
end

function BuffTelegraphs:MakeCategory(lable,form)
		local button = Apollo.LoadForm(self.xmlDoc, "CategoryButton", self.CategoryList, self)
		local menu = Apollo.LoadForm(self.xmlDoc, form, self.OptionsList, self)
		button:GetChildren()[1]:AttachWindow(menu)
		button:GetChildren()[1]:SetText(lable)
		button:GetChildren()[1]:SetCheck(false)
end


function BuffTelegraphs:OnSettingChange()
	self.tSettings.bStalkerMarker = self.PopUp:FindChild("StalkerOptions"):FindChild("IconButton"):IsChecked()
	self.tSettings.bStalkerCircle = self.PopUp:FindChild("StalkerOptions"):FindChild("EnableTelegraph"):IsChecked()
	self.tSettings.nStalkerRange = self.PopUp:FindChild("StalkerOptions"):FindChild("RangeSliderBar"):GetValue()
	self.PopUp:FindChild("StalkerOptions"):FindChild("RangeValue"):SetText(self.tSettings.nStalkerRange)

	self.tSettings.bEsperMarker = self.PopUp:FindChild("EsperOptions"):FindChild("IconButton"):IsChecked()
	self.tSettings.bEsperCircle = self.PopUp:FindChild("EsperOptions"):FindChild("EnableTelegraph"):IsChecked()
	self.tSettings.nEsperRange = self.PopUp:FindChild("EsperOptions"):FindChild("RangeSliderBar"):GetValue()
	self.PopUp:FindChild("EsperOptions"):FindChild("RangeValue"):SetText(self.tSettings.nEsperRange)

	self.tSettings.bPlayerMarker = self.PopUp:FindChild("PlayerOptions"):FindChild("IconButton"):IsChecked()
	self.tSettings.bPlayerCircle = self.PopUp:FindChild("PlayerOptions"):FindChild("EnableTelegraph"):IsChecked()
	self.tSettings.nPlayerRange = self.PopUp:FindChild("PlayerOptions"):FindChild("RangeSliderBar"):GetValue()
	self.PopUp:FindChild("PlayerOptions"):FindChild("RangeValue"):SetText(self.tSettings.nPlayerRange)

	
	
	self.tSettings.bMedicMarker = self.PopUp:FindChild("MedicOptions"):FindChild("IconButton"):IsChecked()
	self.tSettings.bMedicCircle = self.PopUp:FindChild("MedicOptions"):FindChild("EnableTelegraph"):IsChecked()
	self.tSettings.bIgnoreMedicHealers = self.PopUp:FindChild("MedicOptions"):FindChild("IgnoreMedicHealersButton"):IsChecked()
	self.tSettings.nMedicRange = self.PopUp:FindChild("MedicOptions"):FindChild("RangeSliderBar"):GetValue()
	self.PopUp:FindChild("MedicOptions"):FindChild("RangeValue"):SetText(self.tSettings.nMedicRange)

	self.tSettings.bWarriorMarker = self.PopUp:FindChild("WarriorOptions"):FindChild("IconButton"):IsChecked()
	self.tSettings.bWarriorCircle = self.PopUp:FindChild("WarriorOptions"):FindChild("EnableTelegraph"):IsChecked()
	self.tSettings.nWarriorRange = self.PopUp:FindChild("WarriorOptions"):FindChild("RangeSliderBar"):GetValue()
	self.PopUp:FindChild("WarriorOptions"):FindChild("RangeValue"):SetText(self.tSettings.nWarriorRange)

	self.tSettings.bEngineerMarker = self.PopUp:FindChild("EngineerOptions"):FindChild("IconButton"):IsChecked()
	self.tSettings.bEngineerCircle = self.PopUp:FindChild("EngineerOptions"):FindChild("EnableTelegraph"):IsChecked()
	self.tSettings.nEngineerRange = self.PopUp:FindChild("EngineerOptions"):FindChild("RangeSliderBar"):GetValue()
	self.PopUp:FindChild("EngineerOptions"):FindChild("RangeValue"):SetText(self.tSettings.nEngineerRange)
	
	self.tSettings.bSpellslingerMarker = self.PopUp:FindChild("SpellslingerOptions"):FindChild("IconButton"):IsChecked()
	self.tSettings.bSpellslingerCircle = self.PopUp:FindChild("SpellslingerOptions"):FindChild("EnableTelegraph"):IsChecked()
	self.tSettings.nSpellslingerRange = self.PopUp:FindChild("SpellslingerOptions"):FindChild("RangeSliderBar"):GetValue()
	self.PopUp:FindChild("SpellslingerOptions"):FindChild("RangeValue"):SetText(self.tSettings.nSpellslingerRange)

	self.tSettings.bTankMarker = self.PopUp:FindChild("TanksHealersOptions"):FindChild("TankIconButton"):IsChecked()
	self.tSettings.bHealerMarker = self.PopUp:FindChild("TanksHealersOptions"):FindChild("HealerIconButton"):IsChecked()
	
	
	
	

	self.tSettings.cResolution=self.RezSlide:GetValue()
	self.Rez:SetText(self.tSettings.cResolution)
	
	self.tSettings.cPResolution=self.PRezSlide:GetValue()
	self.PRez:SetText(self.tSettings.cPResolution)
	
	self.tSettings.cMinAlpha = self.MinAlphaSlider:GetValue()
	self.MinAlphaValue:SetText(self.tSettings.cMinAlpha.."%")
	
	if (self.tSettings.cCircleAlpha<=self.tSettings.cMinAlpha) then
		self.tSettings.cCircleAlpha = self.tSettings.cMinAlpha + 1
		self.CircleAlphaSlide:SetValue(self.tSettings.cCircleAlpha)
	end
	
	self.tSettings.cCircleAlpha=self.CircleAlphaSlide:GetValue()
	self.CircleAlphaValue:SetText(self.tSettings.cCircleAlpha .."%")
	
	self.tSettings.cMaxCircles=self.CircleLimitSlide:GetValue()
	self.CircleLimit:SetText(self.tSettings.cMaxCircles)
	
	self.tSettings.cMaxIcons=self.IconLimitSlide:GetValue()
	self.IconLimit:SetText(self.tSettings.cMaxIcons)
	
	self.tSettings.cIconSize=self.IconSizeSlide:GetValue()
	self.IconSize:SetText(self.tSettings.cIconSize)

	self.tSettings.cLineThickness = self.LineThicknessSlider:GetValue()
	self.LineThicknessValue:SetText(self.tSettings.cLineThickness .."px")
		
	self.tSettings.cEnabled = self.eButton:IsChecked()
	self.tSettings.frameSkip = self.FrameSkipButton:IsChecked()
	self.tSettings.cShowWarriorBuffIcon = self.ShowWarriorBuffIconButton:IsChecked()
	self.tSettings.cShowMedicBuffIcon = self.ShowMedicBuffIconButton:IsChecked()
	Apollo.StopTimer("SettingChange_CacheTimer")
	Apollo.CreateTimer("SettingChange_CacheTimer", 2, false)
	if not self.tSettings.cEnabled then
		self:Stop()
	end
end

function BuffTelegraphs:SetupOptions()
	self.PopUp:FindChild("StalkerOptions"):FindChild("IconButton"):SetCheck(self.tSettings.bStalkerMarker)
	self.PopUp:FindChild("StalkerOptions"):FindChild("EnableTelegraph"):SetCheck(self.tSettings.bStalkerCircle) 
	self.PopUp:FindChild("StalkerOptions"):FindChild("RangeSliderBar"):SetValue(self.tSettings.nStalkerRange)
	self.PopUp:FindChild("StalkerOptions"):FindChild("RangeValue"):SetText(self.tSettings.nStalkerRange)

	self.PopUp:FindChild("MedicOptions"):FindChild("IconButton"):SetCheck(self.tSettings.bMedicMarker)
	self.PopUp:FindChild("MedicOptions"):FindChild("EnableTelegraph"):SetCheck(self.tSettings.bMedicCircle) 
	self.PopUp:FindChild("MedicOptions"):FindChild("IgnoreMedicHealersButton"):SetCheck(self.tSettings.bIgnoreMedicHealers)

	self.PopUp:FindChild("MedicOptions"):FindChild("RangeSliderBar"):SetValue(self.tSettings.nMedicRange)
	self.PopUp:FindChild("MedicOptions"):FindChild("RangeValue"):SetText(self.tSettings.nMedicRange)

	self.PopUp:FindChild("SpellslingerOptions"):FindChild("IconButton"):SetCheck(self.tSettings.bSpellslingerMarker)
	self.PopUp:FindChild("SpellslingerOptions"):FindChild("EnableTelegraph"):SetCheck(self.tSettings.bSpellslingerCircle) 
	self.PopUp:FindChild("SpellslingerOptions"):FindChild("RangeSliderBar"):SetValue(self.tSettings.nSpellslingerRange)
	self.PopUp:FindChild("SpellslingerOptions"):FindChild("RangeValue"):SetText(self.tSettings.nSpellslingerRange)

	self.PopUp:FindChild("WarriorOptions"):FindChild("IconButton"):SetCheck(self.tSettings.bWarriorMarker)
	self.PopUp:FindChild("WarriorOptions"):FindChild("EnableTelegraph"):SetCheck(self.tSettings.bWarriorCircle) 
	self.PopUp:FindChild("WarriorOptions"):FindChild("RangeSliderBar"):SetValue(self.tSettings.nWarriorRange)
	self.PopUp:FindChild("WarriorOptions"):FindChild("RangeValue"):SetText(self.tSettings.nWarriorRange)

	self.PopUp:FindChild("EngineerOptions"):FindChild("IconButton"):SetCheck(self.tSettings.bEngineerMarker)
	self.PopUp:FindChild("EngineerOptions"):FindChild("EnableTelegraph"):SetCheck(self.tSettings.bEngineerCircle) 
	self.PopUp:FindChild("EngineerOptions"):FindChild("RangeSliderBar"):SetValue(self.tSettings.nEngineerRange)
	self.PopUp:FindChild("EngineerOptions"):FindChild("RangeValue"):SetText(self.tSettings.nEngineerRange)

	self.PopUp:FindChild("EsperOptions"):FindChild("IconButton"):SetCheck(self.tSettings.bEsperMarker)
	self.PopUp:FindChild("EsperOptions"):FindChild("EnableTelegraph"):SetCheck(self.tSettings.bEsperCircle) 
	self.PopUp:FindChild("EsperOptions"):FindChild("RangeSliderBar"):SetValue(self.tSettings.nEsperRange)
	self.PopUp:FindChild("EsperOptions"):FindChild("RangeValue"):SetText(self.tSettings.nEsperRange)

	self.PopUp:FindChild("PlayerOptions"):FindChild("IconButton"):SetCheck(self.tSettings.bPlayerMarker)
	self.PopUp:FindChild("PlayerOptions"):FindChild("EnableTelegraph"):SetCheck(self.tSettings.bPlayerCircle) 
	self.PopUp:FindChild("PlayerOptions"):FindChild("RangeSliderBar"):SetValue(self.tSettings.nPlayerRange)
	self.PopUp:FindChild("PlayerOptions"):FindChild("RangeValue"):SetText(self.tSettings.nPlayerRange)

	
	self.PopUp:FindChild("TanksHealersOptions"):FindChild("TankIconButton"):SetCheck(self.tSettings.bTankMarker)
	self.PopUp:FindChild("TanksHealersOptions"):FindChild("HealerIconButton"):SetCheck(self.tSettings.bHealerMarker)
	
	
	
	
	--Old slider options
	self.TitleMesssage:SetText("BuffTelegraphs V" ..CodeVersion)
		
	self.Rez:SetText(self.tSettings.cResolution)
	self.RezSlide:SetValue(self.tSettings.cResolution)
	
	self.PRez:SetText(self.tSettings.cPResolution)
	self.PRezSlide:SetValue(self.tSettings.cPResolution)

	
	self.CircleLimit:SetText(self.tSettings.cMaxCircles)
	self.CircleLimitSlide:SetValue(self.tSettings.cMaxCircles)
	self.IconLimit:SetText(self.tSettings.cMaxIcons)
	self.IconLimitSlide:SetValue(self.tSettings.cMaxIcons)
	
	self.IconSize:SetText(self.tSettings.cIconSize)
	self.IconSizeSlide:SetValue(self.tSettings.cIconSize)

	self.CircleAlphaValue:SetText(self.tSettings.cCircleAlpha.."%")
	self.CircleAlphaSlide:SetValue(self.tSettings.cCircleAlpha)

	self.MinAlphaValue:SetText(self.tSettings.cMinAlpha.."%")
	self.MinAlphaSlider:SetValue(self.tSettings.cMinAlpha)

	self.LineThicknessValue:SetText(self.tSettings.cLineThickness.."px")
	self.LineThicknessSlider:SetValue(self.tSettings.cLineThickness)

	self.eButton:SetCheck(self.tSettings.cEnabled)

	self.FrameSkipButton:SetCheck(self.tSettings.frameSkip)

	self.ShowWarriorBuffIconButton:SetCheck(self.tSettings.cShowWarriorBuffIcon)
	self.ShowMedicBuffIconButton:SetCheck(self.tSettings.cShowMedicBuffIcon)
	
	self.EmpoweringAuraWindow:SetAnchorOffsets(self.tSettings.cEmpoweringAuraWindowL,self.tSettings.cEmpoweringAuraWindowT,self.tSettings.cEmpoweringAuraWindowR,self.tSettings.cEmpoweringAuraWindowB)

	self.PowerLinkWindow:SetAnchorOffsets(self.tSettings.cPowerLinkWindowL,self.tSettings.cPowerLinkWindowT,self.tSettings.cPowerLinkWindowR,self.tSettings.cPowerLinkWindowB)

	
	self:SetColors()
	
end


function BuffTelegraphs:OnColorChanged(wndHandler, wndControl, crNewColor)
	wndHandler:GetParent():GetData():SetBGColor(crNewColor)
	self:NewColorsSet()
end

function BuffTelegraphs:NewColorsSet()

--	test = self.PopUp:FindChild("EsperColor"):GetBGColor()

	self.tSettings.cEsperColor = self:ColorToStr(self.PopUp:FindChild("EsperColor"):GetBGColor())
	self.tSettings.cStalkerColor = self:ColorToStr(self.PopUp:FindChild("StalkerColor"):GetBGColor())
	self.tSettings.cSpellslingerColor = self:ColorToStr(self.PopUp:FindChild("SpellslingerColor"):GetBGColor())
	self.tSettings.cMedicColor = self:ColorToStr(self.PopUp:FindChild("MedicColor"):GetBGColor())
	self.tSettings.cWarriorColor = self:ColorToStr(self.PopUp:FindChild("WarriorColor"):GetBGColor())		
	self.tSettings.cEngineerColor = self:ColorToStr(self.PopUp:FindChild("EngineerColor"):GetBGColor())
	self.tSettings.cPlayerColor = self:ColorToStr(self.PopUp:FindChild("PlayerColor"):GetBGColor())
	
	self.tSettings.cTankColor = self:ColorToStr(self.PopUp:FindChild("TankColor"):GetBGColor())			
	self.tSettings.cHealerColor = self:ColorToStr(self.PopUp:FindChild("HealerColor"):GetBGColor())

end

function BuffTelegraphs:SetColors()
	self.PopUp:FindChild("EsperColor"):SetBGColor("ff"..self.tSettings.cEsperColor)
	self.PopUp:FindChild("StalkerColor"):SetBGColor("ff"..self.tSettings.cStalkerColor)
	self.PopUp:FindChild("SpellslingerColor"):SetBGColor("ff"..self.tSettings.cSpellslingerColor)
	self.PopUp:FindChild("MedicColor"):SetBGColor("ff"..self.tSettings.cMedicColor)
	self.PopUp:FindChild("WarriorColor"):SetBGColor("ff"..self.tSettings.cWarriorColor)
	self.PopUp:FindChild("EngineerColor"):SetBGColor("ff"..self.tSettings.cEngineerColor)
	self.PopUp:FindChild("PlayerColor"):SetBGColor("ff"..self.tSettings.cPlayerColor)
	
	self.PopUp:FindChild("TankColor"):SetBGColor("ff"..self.tSettings.cTankColor)
	self.PopUp:FindChild("HealerColor"):SetBGColor("ff"..self.tSettings.cHealerColor)
end

--[[
function BuffTelegraphs:ColorToStr(ColorData)
	local crColor = ColorData:ToTable()
	return string.format("%02x%02x%02x", crColor.r, crColor.g, crColor.b)
	return string.format("%02x%02x%02x%02x", a, r, g, b)
	BuffTelegraphs:RGBAToHex(
end
]]

function BuffTelegraphs:ColorToStr(ColorData)
	local crColor = ColorData:ToTable()
	local strColorCode = self:RGBAPercToHex(crColor.r, crColor.g, crColor.b, crColor.a)
	return strColorCode
end

function BuffTelegraphs:RGBAPercToHex(r, g, b, a)
	-- Returns a hexadecimal string for the RGBA values passed.
	if not(a) then a = 1 end
	r = r <= 1 and r >= 0 and r or 0
	g = g <= 1 and g >= 0 and g or 0
	b = b <= 1 and b >= 0 and b or 0
	a = a <= 1 and a >= 0 and a or 1
	-- return hex string
	return string.format("%02x%02x%02x", r*255, g*255, b*255)
end



--- Color Picking Stuff
function BuffTelegraphs:OnColorPick( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY, bDoubleClick, bStopPropagation )
	local defaultColor = "ff"..self:ColorToStr(wndControl:GetBGColor())

	if self.picker then
		self.picker:Destroy()
		self.picker = nil
	end
	
	tCallback = {}
	tCallback.callback = "SetBGColor"
	tCallback.bAlpha = false
	tCallback.bCustomColor = true
	tCallback.strInitialColor = defaultColor
	
	self.picker = GeminiColor:CreateColorPicker(self, tCallback, true, defaultColor,wndControl)
	self:MoveColorPicker()
	self.picker:SetStyle("CloseOnExternalClick",true)
end

function BuffTelegraphs:SetBGColor(strColor, ...)
	arg[3]:SetBGColor(strColor)
	self:RefreshWatchList()
end

function BuffTelegraphs:RefreshWatchList()
	self:Stop()
	self:NewColorsSet()
	for i = 1, #self.watchList do
		if self.watchList[i].marker then
			self.watchList[i].marker:Destroy()
			self.watchList[i].marker = nil
		end

	end

	self.watchList = {}
	if GroupLib.InRaid() or GroupLib.InGroup()then
		Apollo.CreateTimer("RestartTimer", 2, false)
	end

end

function BuffTelegraphs:MoveColorPicker()
	if self.picker ~= nil then
		local pHeight = self.picker:GetHeight()
		local pWidth =self.picker:GetWidth()
		local l,t,r,b = self.PopUp:GetAnchorOffsets()
		local popWidth = self.PopUp:GetWidth()
		self.picker:SetAnchorPoints(0,0,0,0)
		self.picker:SetAnchorOffsets(r-30,t+120,r+pWidth-30,t+pHeight+120)
	end
end


function BuffTelegraphs:ShowEditBuffIcons()
	if self.tSettings.cShowMedicBuffIcon then
		self.EmpoweringAuraWindow:SetStyle("Moveable",true)
		self.EmpoweringAuraWindow:SetStyle("IgnoreMouse", false)
		self.EmpoweringAuraWindow:SetStyle("Sizable", true)
	end
	if self.tSettings.cShowWarriorBuffIcon then
		self.PowerLinkWindow:SetStyle("Moveable",true)
		self.PowerLinkWindow:SetStyle("IgnoreMouse", false)
		self.PowerLinkWindow:SetStyle("Sizable", true)
	end

	self.EmpoweringAuraWindow:Show(self.tSettings.cShowMedicBuffIcon)
	self.PowerLinkWindow:Show(self.tSettings.cShowWarriorBuffIcon)
end

